<?php

namespace Drupal\fieldable_path\Controller;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Url;
use Drupal\field\Entity\FieldConfig;

class FieldablePathController {

  /**
   * Updates path value of the 'fieldable_path' field
   * of the entity of the path source.
   *
   * @param string $source
   *   Path source which is getting updated.
   *   Example: /node/1 or /user/123
   *
   * @param array $path
   *   Array with path information.
   */
  public function updateField($source, array $path = []) {

    // Check if current path source matches any entity.
    $params = Url::fromUri('internal:' . $source)->getRouteParameters();

    // If there's no entity matching the route then we
    // do nothing.
    if (empty($params)) {
      return;
    }

    // Load entity from the path source.
    $entity_type = key($params);
    $entity = \Drupal::entityTypeManager()->getStorage($entity_type)->load($params[$entity_type]);

    // Make sure the current entity exists and contains path field.
    if (empty($entity) || !$entity->hasField('path')) {
      return;
    }

    // Load all 'fieldable_path' fields from entity.
    $fields = $this->getFieldablePathFields($entity);
    $save_entity = FALSE;

    foreach ($fields as $field_name) {
      // There is quite tricky stuff going on here. Normally, we
      // save the field value in field's ::postSave() method,
      // so at this moment field value should already match the
      // path property. However, if someone modifies the path property
      // in the code or outside of entity edit form, we still want to
      // catch it.
      if (!empty($path) && $entity->{$field_name}->value !== $path['alias']) {
        $entity->set($field_name, $path['alias']);
        $save_entity = TRUE;
      }
      elseif (empty($path) && !empty($entity->{$field_name}->value)) {
        $entity->{$field_name} = [];
        $save_entity = TRUE;
      }
    }

    if ($save_entity) {
      // TODO: Is there a good option to save one field only?
      // If you notice that this code makes yet another entity save
      // call during ongoing entity insert/save, then please report an
      // issue, because this is not the desired behavior and potentially
      // dangerous.
      $entity->save();
    }
  }

  /**
   * Returns list of 'fieldable_path' fields
   * for a given entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   Any Drupal entity object.
   *
   * @return array
   *   List of 'fieldable_path' fields.
   */
  public function getFieldablePathFields(EntityInterface $entity) {
    // Get entity fields list.
    // TODO: Is there a more performant way of getting
    // 'fieldable_path' field from an entity? In theory
    // we could get list of all fields of type 'fieldable_path'
    // and check if any of those exist in the current entity.
    $entity_type = $entity->getEntityTypeId();
    $entity_bundle = $entity->bundle();
    $bundle_fields = \Drupal::entityManager()->getFieldDefinitions($entity_type, $entity_bundle);

    // Loop through the fields and searching for added
    // 'fieldable_path' field.
    $fieldable_path_fields = [];
    foreach ($bundle_fields as $field_name => $field_config) {
      if ($field_config instanceof FieldConfig) {
        if ($field_config->getType() == 'fieldable_path') {
          $fieldable_path_fields[] = $field_name;
        }
      }
    }

    return $fieldable_path_fields;
  }
}
